/*
**  AudioList.h
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _Musicbox_MusicBox_MusicBox_H_AudioList
#define _Musicbox_MusicBox_MusicBox_H_AudioList

#include <AppKit/NSView.h>

@class SongInfo;
@class NSTextField;
@class NSButton;
@class NSImage;
@class NSDictionary;
@class NSMutableString;
@class NSString;
@class NSMutableArray;
@class NSArray;

/* This view is used for play list setting.
 * For plain play list, only name of play list
 * is required. For smart playlist as example,
 * more subview may be required.
 */
@interface AudioListView: NSView
{
  NSTextField *nameField;
}

/* Contain all the information to set up a new play list
 * Must contain the "name" key for name of playlist
 */
- (NSDictionary *) property;
- (void) setProperty: (NSDictionary *) property;
@end

@interface AudioList : NSObject
{
  NSMutableArray *listArray;
  NSString *name;
  NSArray *allKeys;
  NSImage *icon;
}

/* The user friendly name of this class */
+ (NSString *) classType;

/* A view used in playlist editor */
+ (AudioListView *) settingView;

- (id) initWithOwner: (id) owner;

/* Read and write data for save.
 * The NSArray contains NSDictionary, which use data of SongInfo as key.
 * Return nil or NO for media which can't read or write data for save,
 * ex. CD-ROM.
 */ 
- (NSArray *) dataForSave;
- (BOOL) loadData: (NSArray *) data;

/* Get property from AudioListView to set up the play list */
- (void) setProperty: (NSDictionary *) property;
- (NSDictionary *) property;

/* Return all keys for filter, ex. Artist, Album, ...
 * The order the keys is corresponding to the path of column in browser.
 * If return nil, the title of browser will use the selection of 
 * preceding column, as general file browser does.
 */
- (NSArray *) allKeys;

/* Return YES when path is not point to the last key 
 * It will be used by browser.
 * When YES, the setLeaf will be NO.
 */
- (BOOL) containMoreKeys: (NSString *) path; 

/* Return an array of SongInfo */
- (NSArray *) soundList;
- (unsigned) count; 

/* Return the filter for browser or sound list for play list.
 * The path should be the same as the order of allKeys.
 * The returned array are autoreleased.
 */
- (NSArray *) filterListWithPath: (NSString *) path;
- (NSArray *) soundListWithPath: (NSString *) path;

/* For Drag and Drop */
- (BOOL) canDragSoundList;
- (BOOL) canDropSoundList;

/* Add object either by drag & drop or by import */
- (BOOL) addObject: (SongInfo *) info;
- (BOOL) addObjects: (NSArray *) array;
- (BOOL) insertObject: (SongInfo *) info atIndex: (int) index;
- (BOOL) deleteObject: (SongInfo *) info;
- (BOOL) removeAllObjects;

/* Icon of play list */
- (NSImage *) icon;
- (void) setIcon: (NSImage *) anImage;

/* Name of play list */
- (NSString *) name;
- (void) setName: (NSString *) theString;

/* Addition button on playlist window,
 * Ex. eject for CD, sync for mp3 player, etc.
 * It will be placed on the right side of window. 
 * return nil if no addition button.
 */
- (NSButton *) additionalButtonOnPlaylistWindow;

/* Addition button on musicbox window */
- (NSButton *) additionalButtonOnMusicBoxWindow;

@end

#endif /* _Musicbox_MusicBox_MusicBox_H_AudioList */
