/*
**  AudioList.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "AudioList.h"
#include <ShengGuang/SongInfo.h>
#include <AppKit/AppKit.h>

@implementation AudioListView

- (id) initWithFrame: (NSRect) frame
{
  NSTextField *nameLabel;

  self = [super initWithFrame: frame];

  nameLabel = [[NSTextField alloc] initWithFrame: NSMakeRect(0, 170, 70, 20)];
  [nameLabel setStringValue: _(@"Source Name:")];
  [nameLabel setEditable: NO];
  [nameLabel setSelectable: NO];
  [nameLabel setBordered: NO];
  [nameLabel setBezeled: NO];
  [nameLabel setBackgroundColor: [NSColor controlBackgroundColor]];
  [self addSubview: nameLabel];
  RELEASE(nameLabel);

  nameField = [[NSTextField alloc] initWithFrame: NSMakeRect(75, 170, 160, 20)];
  [nameField setDelegate: [_window windowController]];
  [nameField setTarget: [_window windowController]];
  [nameField setAction: @selector(okClicked:)];
  [self addSubview: nameField];
  RELEASE(nameField);

  return self;
}

- (NSDictionary *) property
{
  return [NSDictionary dictionaryWithObject: [nameField stringValue]
                                     forKey: @"name"]; 
}

- (void) setProperty: (NSDictionary *) property
{
  [nameField setStringValue: [property objectForKey: @"name"]];
}

@end

@interface AudioList (AudioListPrivate)
- (SongInfo *) songInfoOfFile: (NSString *) filename;
- (NSArray *) artistList;
- (NSArray *) albumList;
- (NSArray *) albumListOfArtist: (NSString *) artist;
- (NSArray *) songListOfArtist: (NSString *) artist album: (NSString *)album;
@end

@implementation AudioList (AudioListPrivate)
- (SongInfo *) songInfoOfFile: (NSString *) filename
{
  return nil;
}

- (NSArray *) albumList
{
  int i;

  id list = [self soundList];
  NSMutableDictionary *dictionary = [NSMutableDictionary new];
  for(i = 0; i < [list count]; i++)
    {
      [dictionary setObject: @"YES"
                   forKey: [[list objectAtIndex: i] album]];

    }
  AUTORELEASE(dictionary);
  return [dictionary allKeys];
}

- (NSArray *) artistList
{
  int i;

  id list = [self soundList];
  NSMutableDictionary *dictionary = [NSMutableDictionary new];
  for(i = 0; i < [list count]; i++)
    {
      [dictionary setObject: @"YES"
                   forKey: [[list objectAtIndex: i] artist]];
      
    }
  AUTORELEASE(dictionary);
  return [dictionary allKeys];
}

- (NSArray *) albumListOfArtist: (NSString *) artist;
{
  int i;

  id list = [self soundList];
  NSMutableDictionary *dictionary = [NSMutableDictionary new];
  for(i = 0; i < [list count]; i++)
    {
      if([[[list objectAtIndex: i] artist] isEqualToString: artist])
        [dictionary setObject: @"YES"
                     forKey: [[list objectAtIndex: i] album]];
    }
  AUTORELEASE(dictionary);
  return [dictionary allKeys];
}

- (NSArray *) songListOfArtist: (NSString *) art album: (NSString *) alb
{
  int i;

  id list = [self soundList];
  NSMutableArray *array = [NSMutableArray new];
  for(i = 0; i < [list count]; i++)
    {
      /* Search artist first */
      if (art) {
        /* Take off non-match */
        if (! [[[list objectAtIndex: i] artist] isEqualToString: art])
          continue;
      } 

      /* Search album */
      if (alb) {
        /* Take off non-match */
        if (! [[[list objectAtIndex: i] album] isEqualToString: alb])
          continue;
      }

      /* Only match both artist and album reach here */
      [array addObject: [list objectAtIndex: i]];
    }
  return AUTORELEASE(array);
}
@end

@implementation AudioList

+ (NSString *) classType
{
  return @"Play List";
}

+ (AudioListView *) settingView
{
  return AUTORELEASE([[AudioListView alloc] initWithFrame: NSMakeRect(5, 50, 240, 180)]);
}

- (NSArray *) allKeys
{
  return allKeys;
}

- (BOOL) containMoreKeys: (NSString *) path
{
  NSArray *array = [path componentsSeparatedByString: @"/"];
  /* path = /Artist/Album */
  if ([array count] > 2)
    return NO;
  return YES;
}

- (NSArray *) filterListWithPath: (NSString *) path
{
  NSMutableArray *list = [NSMutableArray new];
  NSString *unit = nil;

  /* path = @"/", ask artist
   * path = @"/Artist", ask album
   */
  if ([path isEqualToString: @"/"]) 
    {
      [list setArray: [self artistList]];
      if ([list count] > 1)
        unit = _(@"Artists"); 
      else
        unit = _(@"Artist");
    }
  else 
    {
      [list setArray: [self albumListOfArtist: [path lastPathComponent]]];
 
      if ([list count] == 0)
        {
          // It must be all artist
          [list setArray: [self albumList]];
        }
      if ([list count] > 1)
        unit = _(@"Albums");
      else
        unit = _(@"Album");
    }
  if ([list count] == 0)
    {
       RELEASE(list);
       return nil;
    }

  unit = [NSString stringWithFormat: @"%d %@",
                                     [list count],
                                     unit];
  [list insertObject: unit atIndex: 0];
  return AUTORELEASE(list);
}

- (NSArray *) soundListWithPath: (NSString *) path
{
  NSArray *array = [path componentsSeparatedByString: @"/"];
  int index = [array count];

  /* path = /Artist, artist is selected
   * path = /Artist/Album , album is selected
   */
  if (index == 2)
    {
      NSString *artist = [array objectAtIndex: 1];
      NSArray *list = [self songListOfArtist: artist album: nil];
      if ([list count] == 0)
        {
          /* Must be all artist */
          return [self soundList];
        }
      return list;
    }
  if (index == 3)
    {
      NSString *artist = [array objectAtIndex: 1];
      NSString *album = [array objectAtIndex: 2];
      NSArray *allArtists = [self artistList];
      NSArray *allAlbums = [self albumList];
      if([allArtists containsObject: artist] == NO)
        artist = nil;
      if([allAlbums containsObject: album] == NO)
        album = nil;
      return [self songListOfArtist: artist album: album];
    }
  return nil;
}

- (BOOL) canDragSoundList
{
  return YES;
}

- (BOOL) canDropSoundList
{
  return YES;
}

- (BOOL) addObject: (id) object
{
  [listArray addObject: object];
  return YES;
}

- (BOOL) addObjects: (NSArray *) array
{
  [listArray addObjectsFromArray: array];
  return YES;
}

- (BOOL) insertObject: (id) object atIndex: (int) index
{
  [listArray insertObject: object atIndex: index];
  return YES;
} 

- (BOOL) deleteObject: (id) object
{
  [listArray removeObject: object];
  return YES;
}

- (BOOL) removeAllObjects
{
  [listArray removeAllObjects];
  return YES;
}

- (NSArray *) soundList
{
  return listArray;
}

- (unsigned) count
{
  return [[self soundList] count];
}

- (id)initWithOwner: (id)owner
{
  self = [super init];
  listArray = [NSMutableArray new];
  allKeys = [[NSArray alloc] initWithObjects: _(@"Artist"), _(@"Album"), nil];
  [self setName: @"Play List"];
  [self setIcon: [NSImage imageNamed: @"stock_align_justify_16.png"]];
  return self;
}

- (NSImage *) icon
{
  return icon;
}

- (void) setIcon: (NSImage *) theImage
{
  ASSIGN(icon, theImage);
}

- (NSString *) name
{ 
  return name;
}

- (void) setName: (NSString *) theString
{
  ASSIGN(name, theString);
}

- (void) setProperty: (NSDictionary *) dict
{
  if ([dict objectForKey: @"name"])
    [self setName: [dict objectForKey: @"name"]];
}

- (NSDictionary *) property
{
  return nil;
}

- (BOOL) loadData: (NSArray *)data
{
  int i;
  
  for(i = 0; i < [data count]; i++)
    {
      id object = [data objectAtIndex:i]; /* It should be a dictionary */
      SongInfo *info = [[SongInfo alloc] init];
      [info setSong: [object objectForKey: @"song"]];
      [info setArtist: [object objectForKey: @"artist"]];
      [info setAlbum: [object objectForKey: @"album"]];
      [info setFilename: [object objectForKey: @"filename"]];
      [info setDuration: [object objectForKey: @"duration"]];
      [info setChecked: YES]; /* By default, it is checked */
      [self addObject: info];
      RELEASE(info);
    }
  return YES;
}

- (NSArray *) dataForSave
{
  int i;
  NSMutableDictionary *dictionary;

  NSMutableArray *array = [NSMutableArray new];
  id object = [self soundList];

  for(i=0; i < [object count]; i++)
    {
      dictionary = [NSMutableDictionary new];
      [dictionary setObject: [[object objectAtIndex: i] song]
                      forKey: @"song"];
      [dictionary setObject: [[object objectAtIndex: i] artist]
                      forKey: @"artist"];
      [dictionary setObject: [[object objectAtIndex: i] album]
                      forKey: @"album"];
      [dictionary setObject: [[object objectAtIndex: i] filename]
                      forKey: @"filename"];
      [dictionary setObject: [[object objectAtIndex: i] duration]
                      forKey: @"duration"];
      [array addObject: dictionary];
      RELEASE(dictionary);
    }
  return AUTORELEASE(array);
}

- (void)dealloc
{
  RELEASE(name);
  RELEASE(listArray);
  RELEASE(allKeys);
  RELEASE(icon);
  [super dealloc];
}

- (NSButton *) additionalButtonOnPlaylistWindow
{
  return nil;
}

- (NSButton *) additionalButtonOnMusicBoxWindow
{
  return nil;
}

@end
