/*
**  ExtendedTableView.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ExtendedTableView.h"
#include <AppKit/AppKit.h>

@implementation ExtendedTableView

- (void) setButtonColumnIdentifier: (NSString *) string
{
  ASSIGN(buttonColumnIdentifier, string);
}

- (void) dealloc
{
  RELEASE(buttonColumnIdentifier);
  [super dealloc];
}

- (void) mouseDown: (NSEvent *)theEvent
{
  NSPoint location = [theEvent locationInWindow];
  NSTableColumn *tb;

  //
  // Pathological case -- ignore mouse down
  //
  if ((_numberOfRows == 0) || (_numberOfColumns == 0))
    {
      [super mouseDown: theEvent];
      return; 
    }

  // Determine row and column which were clicked
  location = [self convertPoint: location  fromView: nil];
  _clickedRow = [self rowAtPoint: location];
  _clickedColumn = [self columnAtPoint: location];

  if (_clickedRow == -1 || _clickedColumn == -1)
    return;

  tb = [_tableColumns objectAtIndex: _clickedColumn];

  /* Only deal with song column */
  if ([[tb identifier] isEqual: buttonColumnIdentifier])
    {
/*
      NSApplication *theApp = [NSApplication sharedApplication];
      BOOL mouseUp = NO, done = NO;
      NSEvent *e;
      int oldActionMask;
      NSPoint location;
      unsigned int event_mask = NSLeftMouseDownMask | NSLeftMouseUpMask
        | NSMouseMovedMask | NSLeftMouseDraggedMask | NSRightMouseDraggedMask;
*/
      NSCell *mouseCell;
      NSRect mouseCellFrame;
      int state;
      NSRect buttonImageFrame;
      
      mouseCell = [[_tableColumns objectAtIndex: _clickedColumn]
                    dataCellForRow: _clickedRow];

      mouseCellFrame = [self frameOfCellAtColumn: _clickedColumn
                             row: _clickedRow];

      location = [theEvent locationInWindow];
      location = [self convertPoint: location fromView: nil];
      buttonImageFrame = NSMakeRect(mouseCellFrame.origin.x,
                                    mouseCellFrame.origin.y,
                                    mouseCellFrame.size.height,
                                    mouseCellFrame.size.height);

      if (NSMouseInRect(location, buttonImageFrame, YES))
        {
          state = [_dataSource tableView: self
                               stateForTableColumn: tb
                               row: _clickedRow];

          [_dataSource tableView: self
                       setState: !state
                       forTableColumn: tb
                       row: _clickedRow];
        }

      [super mouseDown: theEvent];
      return;

/*

      if (_del_responds)
        {
          [_delegate tableView: self   willDisplayCell: mouseCell 
                     forTableColumn: tb   row: _clickedRow];
        }

      if ([mouseCell isEnabled] == NO)
        return;
      
      if ([mouseCell isContinuous])
        oldActionMask = [mouseCell sendActionOn: 0];
      else
        oldActionMask = [mouseCell sendActionOn: NSPeriodicMask];
      
      // [_window _captureMouse: self];

      NSLog(@"modifierFlags %d", [theEvent modifierFlags]);

      e = theEvent;
      while (!done)                 // loop until mouse goes up
        {
          location = [e locationInWindow];
          location = [self convertPoint: location fromView: nil];
          // ask the cell to track the mouse only
          // if the mouse is within the cell
          if (NSMouseInRect(location, mouseCellFrame, YES))
            {
              [mouseCell highlight: YES 
                         withFrame: mouseCellFrame 
                         inView: self];
              [_window flushWindow];
              if ([mouseCell trackMouse: e
                             inRect: mouseCellFrame 
                             ofView: self
                             untilMouseUp: NO])
                done = mouseUp = YES;
              else
                {
                  [mouseCell highlight: NO 
                             withFrame: mouseCellFrame
                             inView: self];
                  [_window flushWindow];
                }
            }
          
          if (done)
            break;
          
          e = [theApp nextEventMatchingMask: event_mask
                      untilDate: nil
                      inMode: NSEventTrackingRunLoopMode
                      dequeue: YES];
          if ([e type] == NSLeftMouseUp)
            done = YES;
        }
      
      // [_window _releaseMouse: self];
      if (mouseUp)
        {
          [_dataSource tableView: self
                        setState: [mouseCell state]
                  forTableColumn: tb
                             row: _clickedRow];

          [mouseCell highlight: NO 
                     withFrame: mouseCellFrame
                     inView: self];

          [_window flushWindow];
        }
      
      [mouseCell sendActionOn: oldActionMask];
      
      if (mouseUp)
        {
          SEL theAction;
          theAction = [mouseCell action];
          if (theAction)
            [NSApp sendAction: theAction
                   to: [mouseCell target]
                   from: mouseCell];
        }
      return;
*/
    }
  
  [super mouseDown: theEvent];
}

- (void)drawRow: (int)rowIndex clipRect: (NSRect) clipRect
{
  int startingColumn;
  int endingColumn;
  NSTableColumn *tb;
  NSRect drawingRect;
  id cell;
  int i;
  float x_pos;

  if (_dataSource == nil)
    {
      return;
    }

  /* Using columnAtPoint: here would make it called twice per row per drawn
     rect - so we avoid it and do it natively */

  /* Determine starting column as fast as possible */
  x_pos = NSMinX (clipRect);
  i = 0;
  while ((x_pos > _columnOrigins[i]) && (i < _numberOfColumns))
    {
      i++;
    }
  startingColumn = (i - 1);

  if (startingColumn == -1)
    startingColumn = 0;

  /* Determine ending column as fast as possible */
  x_pos = NSMaxX (clipRect);
  // Nota Bene: we do *not* reset i
  while ((x_pos > _columnOrigins[i]) && (i < _numberOfColumns))
    {
      i++;
    }
  endingColumn = (i - 1);

  if (endingColumn == -1)
    endingColumn = _numberOfColumns - 1;

  /* Draw the row between startingColumn and endingColumn */
  for (i = startingColumn; i <= endingColumn; i++)
    {
      if (i != _editedColumn || rowIndex != _editedRow)
        {
          tb = [_tableColumns objectAtIndex: i];
          cell = [tb dataCellForRow: rowIndex];
          if (_del_responds)
            {
              [_delegate tableView: self   willDisplayCell: cell
                         forTableColumn: tb   row: rowIndex];
            }
     
          [cell setObjectValue: [_dataSource tableView: self
                                             objectValueForTableColumn: tb
                                             row: rowIndex]];

          if ([cell isKindOfClass: [NSButtonCell class]])
            [cell setState: [_dataSource tableView: self
                                         stateForTableColumn: tb
                                         row: rowIndex]];

          drawingRect = [self frameOfCellAtColumn: i
                              row: rowIndex];

          [cell drawWithFrame: drawingRect inView: self];
        }
    }
}

@end
