/*
**  ModuleLoader.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ModuleLoader.h"
#include <Foundation/NSBundle.h>
#include <Foundation/NSPathUtilities.h>
#include <Foundation/NSArray.h>
#include <Foundation/NSFileManager.h>

static ModuleLoader *defaultLoader = nil;

NSString *MusicBoxLocalLibraryPath()
{
  return [NSString stringWithFormat: @"%@/MusicBox", 
                      [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, 
                                                       NSLocalDomainMask, YES)
                                                          objectAtIndex:0] ];
}

NSString *BundlesLocalLibraryPath()
{
  return [NSString stringWithFormat: @"%@/Bundles",
                      [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, 
                                                      NSLocalDomainMask, YES)
                                                          objectAtIndex:0] ];

}

@implementation ModuleLoader

+ (id) defaultLoader
{
  if ( !defaultLoader)
    {
      defaultLoader = [[ModuleLoader alloc] init];
    }
  return defaultLoader;
}

//
// Return all modules name with specific extension
// in $GNUSTEP_USER_ROOT/Library/MusicBox
// The module name is the path of the module
//
- (NSArray *) allModulesNames: (NSString *) extension
{
  NSArray *allFiles;
  NSMutableArray *allNames;
  NSString *string;
  int i;

  NSFileManager *fileManager = [NSFileManager defaultManager];
  allNames = [NSMutableArray new];

  /* Search for GNUstep/Library/MusicBox/ */
  allFiles = [fileManager directoryContentsAtPath: MusicBoxLocalLibraryPath()];

  for (i = 0; i < [allFiles count]; i++)
    {
      string = [allFiles objectAtIndex: i];
      
      // If we found a bundle and the extension is equal, then load it!
      if ( [[string pathExtension] isEqualToString: extension] )
        {
          [allNames addObject: string];
        }
    }

  /* Search for GNUstep/Library/Bundles/ */
  allFiles = [fileManager directoryContentsAtPath: BundlesLocalLibraryPath()];

  for (i = 0; i < [allFiles count]; i++)
    {
      string = [allFiles objectAtIndex: i];
     
      // If we found a bundle and the extension is equal, then load it!
      if ( [[string pathExtension] isEqualToString: extension] )
        {
          [allNames addObject: string];
        }
    }

  return AUTORELEASE(allNames);
}

// Return the Class name of a specific module
- (Class) classNameOf: (NSString *) module
{
  NSBundle *bundle;
  Class bundleClass;

  /* Try GNUstep/Library/MusicBox */
  NSString *path = [NSString stringWithFormat: @"%@/%@",
                                     MusicBoxLocalLibraryPath(),
                                     module];
  
  bundle = [NSBundle bundleWithPath: path];

  if (bundle)
    {
      bundleClass = [bundle principalClass];

      if ( bundleClass )
        {
          return bundleClass;
        }
      else
        {
          NSLog(@"Failed to get Class from module %@", module);
          return nil;
        }
    }
  /* Try GNUstep/Library/Bundles */
  path = [NSString stringWithFormat: @"%@/%@",
                                     BundlesLocalLibraryPath(),
                                     module];
 
  bundle = [NSBundle bundleWithPath: path];

  if (bundle)
    {
      bundleClass = [bundle principalClass];

      if ( bundleClass )
        {
          return bundleClass;
        }
      else
        {
          NSLog(@"Failed to get Class from module %@", module);
          return nil;
        }
    }

  NSLog(@"ModuleLoader: Error loading bundle at path %@", module);
  return nil;
}

@end
